
#ifndef _PATTERN_H_
#define _PATTERN_H_

// As simple as a linked list can get, really.
struct pattern_list;
typedef struct pattern_list {
	int value;
	struct pattern_list *next_item;
} pattern_list;

// As simple as a tree can get, really. For fast searching.
struct pattern_tree;
typedef struct pattern_tree {
	int value;
	struct pattern_tree *next_sibling;
	struct pattern_tree *first_child;
} pattern_tree;

// A way to make one malloc/free instead of a dozen.
typedef struct pattern_buf {
	unsigned char *buf;
	int pos;
	int size;
} pattern_buf;

pattern_list *new_pattern_list(unsigned char *buf, int value);
void pattern_list_destroy(pattern_list *list);
void pattern_list_add(pattern_list *list, pattern_list *item);
void pattern_list_pop(pattern_list *list, int index);

pattern_tree *new_pattern_tree(unsigned char *buf, int value);
void pattern_tree_destroy(pattern_tree *tree);
int pattern_tree_compare(pattern_tree *tree, pattern_list *list, pattern_tree **match);
void pattern_tree_add_child(pattern_tree *tree, pattern_tree *child);

// Find just on this level.
pattern_tree *pattern_tree_find(pattern_tree *tree, int value);

// Add to a tree with a null-terminated string.
int pattern_tree_input(pattern_tree *tree, char *tree_input, pattern_buf *buf);

int new_pattern_buf(pattern_buf *buf, int size);
void pattern_buf_destroy(pattern_buf *buf);
unsigned char *pattern_buf_get(pattern_buf *buf, int size);

#endif // _PATTERN_H_
